window.Gleam.initSearch([{"doc":"formz","title":"formz","content":"# formz\n\n[![Package Version](https://img.shields.io/hexpm/v/formz)](https://hex.pm/packages/formz)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/formz/)\n\nA Gleam library for parsing and generating accessible HTML forms.\n\nHTML forms rendered in the browser and the data they are parsed into are\nintrinsically linked. Treating the markup and the parsing as two separate\nproblems to solve is inconvenient and leads to bugs. This library aims\nto make that link explicit and easy to manage, while making it really easy\nto make accessible forms.\n\nNote: This library is not necessarily well-suited for generating one-off\nforms, and is intended for use in projects where you have a few forms to\nmanage, and would like to keep the form markup and parsing logic in sync.  It\ntakes some amount of effort to make an actual form generator with markup and\nstyles, and that might not be worth it for a one-off form. That said, a simple\nform generator is provided if you aren&#39;t opinionated about your markup.\n\n```sh\ngleam add formz@0.1\n```\n\n## Creating a form\n\nA `formz` form is a list of fields and a decoder function.  You construct the\ndecoder function as fields are added:\n\n```gleam\nimport formz\nimport formz_string/definition\n\npub fn make_form() {\n  use username &lt;- formz.field(formz.named(&quot;username&quot;), definition.text_field())\n  use password &lt;- formz.field(formz.named(&quot;password&quot;), definition.password_field())\n\n  formz.create_form(#(username, password))\n}\n```\n\n## Creating fields\n\nThere are two arguments to adding a field to a form (seen above):\n\n1. A `Config`, which holds specific, unique details about the field: its name,\n   label, help text, and disabled state.\n2. A `Definition`, which says (A) how to generate the HTML input element for\n   the field, and (B) how to parse the data from the field. These definitions\n   are reusable and can be shared across fields, forms and projects.\n\n### Field config\n\n```gleam\n// name is required, the other confg are optional\nformz.named(&quot;username&quot;)\n|&gt; formz.set_label(&quot;Username&quot;)\n|&gt; formz.set_help_text(&quot;Only alphanumeric characters are allowed.&quot;)\n```\n\n```gleam\nformz.named(&quot;userid&quot;) |&gt; formz.make_disabled\n```\n\n### Field definition\n\nA `Definition` describes how an input works, e.g. how it looks and how it&#39;s\nparsed. Definitions are intended to be reusable.\n\nThe first role of a `Defintion` is to generate the HTML input for the field.\nThis library is format-agnostic and you can generate inputs as raw\nstrings, Lustre elements, Nakai nodes, something else, etc. The second role\nof a `Definition` is to parse the raw string data from the input into a\nGleam type.\n\nThere are currently three `formz` libraries that provide common field\ndefinitions for the most common HTML inputs:\n\n- [formz_string](https://hexdocs.pm/formz_string/)\n- [formz_nakai](https://hexdocs.pm/formz_nakai/)\n- [formz_lustre](https://hexdocs.pm/formz_lustre/)\n\n```gleam\n/// you won&#39;t often need to do this directly (I think??).  The idea is that\n/// there&#39;d be libs with the definitions you need.\n\nimport formz\nimport formz/validation\nimport lustre/attribute\nimport lustre/element\nimport lustre/element/html\n\nfn password_widget(\n  config: formz.Config,\n  state: formz.InputState,\n) -&gt; element.Element(msg) {\n  html.input([\n    attribute.type_(&quot;password&quot;),\n    attribute.name(config.name),\n    attribute.id(config.name),\n    attribute.attribute(&quot;aria-labelledby&quot;, config.label),\n  ])\n}\n\npub fn password_field() {\n  definition(\n    widget: password_widget,\n    parse: validation.string,\n    // We need to have a stub value for each definition. The stubs are used when\n    // building the decoder functions for the form. This is just any value of\n    // the same type that the parse function returns.\n    stub: &quot;&quot;,\n  )\n}\n```\n\n\n\n## Generating HTML for a form\n\nGenerally speaking, the idea with a `formz` form is that you are not going\nto generate the HTML for each field individually, but rather, you&#39;d use\na function to loop through each field, generating semantic, accessible\nmarkup for each one.\n\nThe specifics of how you would do this are going to vary greatly for each\nproject and its styling/markup needs.\n\nHowever, the three `formz_*` libraries mentioned above all provide a\nsimple form generator function that you can use as is, or as a starting\npoint for your own.  `formz` is BYOS, Bring Your Own Stylesheet, so the\nbuilt-in form generators come unstyled. If there is interest, I could add\na super simple CSS file to get the ball rolling and make the default\nforms easier to use out of the box.\n\nThat said, you can also create the form HTML yourself, directly for each field.\nThere&#39;s [an example](https://github.com/bentomas/formz/blob/main/formz_demo/src/formz_demo/examples/custom_output.gleam)\nin the demo project showing how to do this.\n\n### Generating form HTML using the `formz_string` library\n\nThe built-in form generators leave it as homework to add the form tags and\nsubmit buttons.\n\n```gleam\nimport formz_string/simple\n\npub fn show_form(form) -&gt; String {\n  &quot;&lt;form method=\\&quot;post\\&quot;&gt;&quot;\n  &lt;&gt; simple.generate(form)\n  &lt;&gt; &quot;&lt;p&gt;&lt;button type\\&quot;submit\\&quot;&gt;Submit&lt;/button&gt;&lt;/p&gt;&quot;\n  &lt;&gt; &quot;&lt;/form&gt;&quot;\n}\n```\n\n\n## Parsing form data\n\nYou can parse a `formz` form with a tuple of values and names, typically from\na POST request.  Here we parse in a `wisp` handler:\n\n```gleam\npub fn handle_form_submission(req: Request) -&gt; Response {\n  use formdata &lt;- wisp.require_form(req)\n\n  let result = make_form()\n  |&gt; formz.data(formdata.values)\n  |&gt; formz.parse\n\n  case result {\n    Ok(credentials) -&gt; {\n      let #(username, password) = credentials\n      wisp.ok()\n      |&gt; wisp.html_body(string_builder.from_string(&quot;Hello &quot;&lt;&gt;username&lt;&gt;&quot;!&quot;))\n    }\n    Error(form_with_errors) -&gt; {\n      show_form(form_with_errors)\n    }\n  }\n}\n```\n\nHowever, often you want to parse a form, and then... you know... act on that\ndata, and in doing so you might discover more errors for the form.  In this\nsituation you can use `decode_then_try`:\n\n```gleam\npub fn handle_form_submission(req: Request) -&gt; Response {\n  use formdata &lt;- wisp.require_form(req)\n\n  let result = make_form()\n  |&gt; formz.data(formdata.values)\n  |&gt; formz.decode_then_try(fn(form, credentials) {\n    case credentials {\n      #(&quot;admin&quot; as username, &quot;l33t&quot;) -&gt; Ok(username)\n      #(&quot;admin&quot;, _) -&gt;\n        Error(form |&gt; formz.field_error(&quot;password&quot;, &quot;Wrong password&quot;))\n      _ -&gt;\n        Error(form |&gt; formz.field_error(&quot;username&quot;, &quot;Unknown username&quot;))\n    }\n  })\n\n  case result {\n    Ok(username) -&gt; {\n      wisp.ok()\n      |&gt; wisp.html_body(string_builder.from_string(&quot;Hello &quot; &lt;&gt; username &lt;&gt; &quot;!&quot;))\n    }\n    Error(form_with_errors) -&gt; {\n      show_form(form_with_errors)\n    }\n  }\n}\n```\n\n## See it in action\n\nThere is a [demo wisp app](https://github.com/bentomas/formz/tree/main/formz_demo)\nshowing a few interactive examples of how `formz` works in the repo.\n","url":"index.html"},{"doc":"formz","title":"Config","content":"pub type Config {\n  Config(\n    name: String,\n    label: String,\n    help_text: String,\n    disabled: Bool,\n  )\n}\n Configuration for the form `Item`.  The only required information here is\n `name` and there is a `named` function to make a config with a name.  You\n can then chain the `set_x` functions to add the other information as needed.\n\n ### Example\n\n \n let config = formz.named(&quot;name&quot;) |&gt; formz.set_label(&quot;Full Name&quot;)\n \n\n \n let config =\n   formz.named(&quot;name&quot;)\n   |&gt; formz.set_label(&quot;Full Name&quot;)\n   |&gt; formz.set_help_text(&quot;Enter your full name&quot;)\n \n\n \n let config = formz.named(&quot;name&quot;) |&gt; formz.make_disabled\n \nConfig(\n  name: String,\n  label: String,\n  help_text: String,\n  disabled: Bool,\n)\n\nname\n&lt;p&gt;The name of the field or subform. The only truly required information\nfor a form &lt;code&gt;Item&lt;/code&gt;. This is used to identify the field in the form.\nIt should be unique for each form, and is untested with any values other\nthan strings solely consisting of alphanumeric characters\nand underscores.&lt;/p&gt;\n\nlabel\n&lt;p&gt;This library thinks of a label as required, but will make one for you from\nthe name if you don’t provide one via the &lt;code&gt;field&lt;/code&gt; function. For\naccessibility reasons, a field should always provide a label and all\nthe maintained form generators will output one.&lt;/p&gt;\n\nhelp_text\n&lt;p&gt;Optional help text for the field. This is used to provide additional\ninstructions or context for the field.  It is up to the form generator\nto decide if and how to display this text.&lt;/p&gt;\n\ndisabled\n&lt;p&gt;Whether the field is disabled. A disabled field is not editable in\nthe browser.  However, there is nothing stopping a user from changing\nthe value or submitting a different value via other means, so (presently)\nthis doesn’t mean the value cannot be tampered with.&lt;/p&gt;\n\nSynonyms:\nformz.Config\nformz Config","url":"formz.html#Config"},{"doc":"formz","title":"Definition","content":"pub opaque type Definition(widget, required, optional)\n A `Definition` describes how an input works, e.g. how it looks and how it&#39;s\n parsed. Definitions are intended to be reusable.\n\n The first role of a `Defintion` is to generate the HTML input for the field.\n This library is format-agnostic and you can generate inputs as raw\n strings, Lustre elements, Nakai nodes, something else, etc. The second role\n of a `Definition` is to parse the raw string data from the input into a\n Gleam type.\n\n There are currently three `formz` libraries that provide common field\n definitions for the most common HTML inputs:\n\n - [formz_string](https://hexdocs.pm/formz_string/)\n - [formz_nakai](https://hexdocs.pm/formz_nakai/)\n - [formz_lustre](https://hexdocs.pm/formz_lustre/)\n\n How a definition parses an input value depends on whether a value is required\n for that input (i.e.  whether `field`, `required_field`, `list`, or `limited_list`\n was used to add it to the form).  If a value is required, the definition is\n expected to return a string error if the input is empty, or the `required` type\n if it isn&#39;t.  You can use the `definition` function to create a simple\n definition that just parses to an `Option` if an input is empty.\n\n However, not all fields should be parsed into an `Option` when\n given an empty input value. For example, an optional text field might be an\n empty string or an optional checkbox might be False.  For these cases, you\n can use the `definition_with_custom_optional` function to create a definition\n that can parse to any type when the input is empty.\n\nSynonyms:\nformz.Definition\nformz Definition","url":"formz.html#Definition"},{"doc":"formz","title":"Form","content":"pub opaque type Form(widget, output)\n You create this using the `create_form` function.\n\n The `widget` type is set by the `Definition`s used to add fields for\n this form, and has the details of how to turn given fields into HTML inputs.\n\n The `output` type is the type of the decoded data from the form. This is\n set directly by the `create_form` function, after all the fields have\n been added.\n\nSynonyms:\nformz.Form\nformz Form","url":"formz.html#Form"},{"doc":"formz","title":"InputState","content":"pub type InputState {\n  Unvalidated(value: String, requirement: Requirement)\n  Valid(value: String, requirement: Requirement)\n  Invalid(value: String, requirement: Requirement, error: String)\n}\n The state of the an input for a field. This is used to track the current\n raw value,  whether a value is required or not, if the value has been\n validated, and the outcome of that validation.\nUnvalidated(value: String, requirement: Requirement)\n\n\nValid(value: String, requirement: Requirement)\n\n\nInvalid(value: String, requirement: Requirement, error: String)\n\n\nSynonyms:\nformz.InputState\nformz InputState","url":"formz.html#InputState"},{"doc":"formz","title":"Item","content":"pub type Item(widget) {\n  Field(config: Config, state: InputState, widget: widget)\n  ListField(\n    config: Config,\n    states: List(InputState),\n    limit_check: LimitCheck,\n    widget: widget,\n  )\n  SubForm(config: Config, items: List(Item(widget)))\n}\n You add an `Item` to a form using the `field`, `required_field`, `list`,\n `limited_list` and `subform` functions. A form is a list of `Item`s and\n each item is parsed to a single value, and then passed to the decode\n function.\n\n You primarily only use an `Item` directly when writing a form generator\n function to output your form to HTML.\n\n You can also manipulate an `Item` after a form has been created, to change\n things like labels, help text, etc with `update`. Each item has a `Config`\n that describes how it works, and this can be updated directly\n with `update_config`.\n\n \n let form = make_form()\n formz.update_config(&quot;name&quot;, formz.set_label(_, &quot;Full Name&quot;))\n \nField(config: Config, state: InputState, widget: widget)\n A single field that (generally speaking) corresponds to a single\n HTML input\n\nListField(\n  config: Config,\n  states: List(InputState),\n  limit_check: LimitCheck,\n  widget: widget,\n)\n A single field that a consumer can submit multiple values for.\n\nSubForm(config: Config, items: List(Item(widget)))\n A group of fields that are added as and then parsed to a single unit.\n\nSynonyms:\nformz.Item\nformz Item","url":"formz.html#Item"},{"doc":"formz","title":"LimitCheck","content":"pub type LimitCheck =\n  fn(Int) -&gt; Result(List(InputState), Int)\n When adding a list field to a form with `limited_list`, you have to provide\n a `LimitCheck` function that checks the number of inputs (and associated\n values) for the field. This function can either say the number of inputs\n is Ok and optionally add more, or say the number of inputs was too high\n and return an error. For example, you are presenting a blank form to a\n consumer, and you want to show three initial fields for them to fill out,\n or you want to always show one more additional field than the number of\n values that have already belong to the form, etc.\n\n There are helper functions, `limit_at_least`, `limit_at_most`, and\n `limit_between` or more generally `simple_limit_check` to make a\n `LimitCheck` function for you.  I would imagine that those will cover 99.9%\n of cases and almost no one will need to write their own `LimitCheck`.  But\n if you do, look at the source for `simple_limit_check` for a better idea\n of how to write one.\n\n This function takes as its only argument, the number of fields that already\n have a value.  It should return either `Ok` with a list of `Unvalidated`\n `InputState` items if it wants to offer more inputs to consumers of the\n form, or `Error` amount of inputs that were too many.\n\n This is used multiple times... when the form is created so we know how many\n initial inputs to present, when data is added so we know if we need to add\n more inputs so users can add more items, and when the form is decoded and\n we are checking if too many fields have been added.\n\nSynonyms:\nformz.LimitCheck\nformz LimitCheck","url":"formz.html#LimitCheck"},{"doc":"formz","title":"Requirement","content":"pub type Requirement {\n  Optional\n  Required\n}\n Whether an input value is required for an input field.\nOptional\n\n\nRequired\n\n\nSynonyms:\nformz.Requirement\nformz Requirement","url":"formz.html#Requirement"},{"doc":"formz","title":"create_form","content":"pub fn create_form(thing: a) -&gt; Form(b, a)\n Create an empty form that &quot;decodes&quot; directly to `thing`. This is\n intended to be the final return value of a chain of callbacks that adds\n the form&#39;s fields.\n\n \n create_form(1)\n |&gt; decode\n # -&gt; Ok(1)\n \n \n fn make_form() {\n   use field1 &lt;- formz.required_field(field(&quot;field1&quot;), definitions.text_field())\n   use field2 &lt;- formz.required_field(field(&quot;field2&quot;), definitions.text_field())\n   use field3 &lt;- formz.required_field(field(&quot;field3&quot;), definitions.text_field())\n\n   formz.create_form(#(field1, field2, field3))\n }\n \nSynonyms:\nformz.create_form\nformz create_form","url":"formz.html#create_form"},{"doc":"formz","title":"data","content":"pub fn data(\n  form: Form(a, b),\n  input_data: List(#(String, String)),\n) -&gt; Form(a, b)\n Add input data to this form. This will set the raw string value of the fields.\n It does not trigger any parsing or decoding, so you can also use this to set\n default values (if you do it in your form generator function) or initial values\n (if you do it before rendering a blank form).\n\n The input data is a list of tuples, where the first element is the name of the\n field and the second element is the value to set.  If the field does not exist\n the data is ignored.\n\n This resets the validation state of the fields that have data, so you&#39;ll need to\n re-validate or decode the form after setting data.\nSynonyms:\nformz.data\nformz data","url":"formz.html#data"},{"doc":"formz","title":"decode","content":"pub fn decode(form: Form(a, b)) -&gt; Result(b, Form(a, b))\n Decode the form.  This means step through the fields one by one, parsing\n them individually.  If any field fails to parse, the whole form is considered\n invalid, however it will still continue parsing the rest of the fields to\n collect all errors.  This is useful for showing all errors at once.  If no\n fields fail to parse, the decoded value is returned, which is the value given\n to `create_form`.\n\n If you&#39;d like to decode the form but not get the output, so you can give\n feedback to a user in response to input, you can use `validate` or `validate_all`.\nSynonyms:\nformz.decode\nformz decode","url":"formz.html#decode"},{"doc":"formz","title":"decode_then_try","content":"pub fn decode_then_try(\n  form: Form(a, b),\n  apply fun: fn(Form(a, b), b) -&gt; Result(c, Form(a, b)),\n) -&gt; Result(c, Form(a, b))\n Decode the form, then apply a function to the output if it was successful.\n This is a very thin wrapper around `decode` and `result.try`, but the\n difference being it will pass the form along to the function as a second\n argument in addition to the successful result.  This allows you to easily\n update the form fields with errors or other information based on the output.\n\n This is useful for situations where you can have errors in the form that\n aren&#39;t easily checked in simple parsing functions.  Like, say, hitting a\n db to check if a username is taken.\n\n As a reminder, parse functions will be called multiple times for each field\n when the form is being made, validated and parsed, and should not contain\n side effects.  This function is the proper way to add errors to fields from\n functions that have side effects.\n\n \n make_form()\n |&gt; data(form_data)\n |&gt; decode_then_try(fn(username, form) {\n   case is_username_taken(username) {\n     Ok(false) -&gt; Ok(form)\n     Ok(true) -&gt; field_error(form, &quot;username&quot;,  &quot;Username is taken&quot;)\n   }\n }\nSynonyms:\nformz.decode_then_try\nformz decode_then_try","url":"formz.html#decode_then_try"},{"doc":"formz","title":"definition","content":"pub fn definition(\n  widget widget: a,\n  parse parse: fn(String) -&gt; Result(b, String),\n  stub stub: b,\n) -&gt; Definition(a, b, Option(b))\n Create a simple `Definition` that is parsed as an `Option` if the field\n is empty.  See [formz_string](https://hexdocs.pm/formz_string/formz_string/definitions.html)\n for more examples of making widgets and definitions.\nSynonyms:\nformz.definition\nformz definition","url":"formz.html#definition"},{"doc":"formz","title":"definition_with_custom_optional","content":"pub fn definition_with_custom_optional(\n  widget widget: a,\n  parse parse: fn(String) -&gt; Result(b, String),\n  stub stub: b,\n  optional_parse optional_parse: fn(\n    fn(String) -&gt; Result(b, String),\n    String,\n  ) -&gt; Result(c, String),\n  optional_stub optional_stub: c,\n) -&gt; Definition(a, b, c)\n Create a `Definition` that can parse to any type if the field is optional.\n This takes two functions.  The first, `parse`, is the &quot;required&quot; parse\n function, which takes the raw string value, and turns it into the required\n type.  The second, `optional_parse`, is a function that takes the normal\n parse function and the raw string value, and it is supposed to check the\n input string: if it is empty, return an `Ok` with a value of the optional\n type; and if it&#39;s not empty use the normal parse function.\n\n See [formz_string](https://hexdocs.pm/formz_string/formz_string/definitions.html)\n for more examples of making widgets and definitions.\nSynonyms:\nformz.definition_with_custom_optional\nformz definition_with_custom_optional","url":"formz.html#definition_with_custom_optional"},{"doc":"formz","title":"field","content":"pub fn field(\n  config: Config,\n  definition: Definition(a, b, c),\n  next: fn(c) -&gt; Form(a, d),\n) -&gt; Form(a, d)\n Add an optional field to a form.\n\n Ultimately whether a field is actually optional or not comes down to the\n details of the definition.  The definition will receive the raw input string\n and is in charge of returning an error or an optional value.\n\n If multiple values are submitted for this field, the last one will be used.\n\n The final argument is a callback that will be called when the form\n is being... constructed to look for more fields; validated to check for\n errors; and decoded to parse the input data.  **For this reason, the\n callback should be a function without side effects.** It can be called any\n number of times. Don&#39;t do anything but create the type with the data you\n need!  If you need to do decoding that has side effects, you should use\n `decode_then_try`.\nSynonyms:\nformz.field\nformz field","url":"formz.html#field"},{"doc":"formz","title":"field_error","content":"pub fn field_error(\n  form: Form(a, b),\n  name: String,\n  str: String,\n) -&gt; Form(a, b)\n Convenience function for setting the `InputState` of a field to\n Invalid with a given error message.\n\n ### Example\n\n \n field_error(form, &quot;username&quot;,  &quot;Username is taken&quot;)\n \nSynonyms:\nformz.field_error\nformz field_error","url":"formz.html#field_error"},{"doc":"formz","title":"get","content":"pub fn get(\n  form: Form(a, b),\n  name: String,\n) -&gt; Result(Item(a), Nil)\n Get the [`Item`](https://hexdocs.pm/formz/formz.html#Item) with the\n given name.  If multiple items have the same name, the first one is returned.\nSynonyms:\nformz.get\nformz get","url":"formz.html#get"},{"doc":"formz","title":"items","content":"pub fn items(form: Form(a, b)) -&gt; List(Item(a))\n Get each [`Item`](https://hexdocs.pm/formz/formz.html#Item) added\n to the form.  Any time a field, list field, or subform are added, a `Item`\n is created.  Use this to loop through all the fields of your form and\n generate HTML for them.\nSynonyms:\nformz.items\nformz items","url":"formz.html#items"},{"doc":"formz","title":"limit_at_least","content":"pub fn limit_at_least(\n  min: Int,\n) -&gt; fn(Int) -&gt; Result(List(InputState), Int)\n Convenience function for creating a `LimitCheck` with a minimum number\n of required values.  This sets the maximum to `1,000,000`, effectively unlimited.\nSynonyms:\nformz.limit_at_least\nformz limit_at_least","url":"formz.html#limit_at_least"},{"doc":"formz","title":"limit_at_most","content":"pub fn limit_at_most(\n  max: Int,\n) -&gt; fn(Int) -&gt; Result(List(InputState), Int)\n Convenience function for creating a `LimitCheck` with a maximum number\n of accepted values.  This sets the minimum to `0`.\nSynonyms:\nformz.limit_at_most\nformz limit_at_most","url":"formz.html#limit_at_most"},{"doc":"formz","title":"limit_between","content":"pub fn limit_between(\n  min: Int,\n  max: Int,\n) -&gt; fn(Int) -&gt; Result(List(InputState), Int)\n Convenience function for creating a `LimitCheck` with a minimum and maximum\n number of values.\nSynonyms:\nformz.limit_between\nformz limit_between","url":"formz.html#limit_between"},{"doc":"formz","title":"limited_list","content":"pub fn limited_list(\n  limit_check: fn(Int) -&gt; Result(List(InputState), Int),\n  config: Config,\n  is definition: Definition(a, b, c),\n  next next: fn(List(b)) -&gt; Form(a, d),\n) -&gt; Form(a, d)\n Add a list field to a form, but with limits on the number of values that\n can be submitted.  The `limit_check` function is used to impose those\n limits, and the `limit_at_least`, `limit_at_most`, and `limit_between`\n functions help you create this function for the most likely scenarios.\n\n The final argument is a callback that will be called when the form\n is being... constructed to look for more fields; validated to check for\n errors; and decoded to parse the input data.  **For this reason, the\n callback should be a function without side effects.** It can be called any\n number of times. Don&#39;t do anything but create the type with the data you\n need!  If you need to do decoding that has side effects, you should use\n `decode_then_try`.\n\n### Example\n\n \n fn make_form() {\n  use names &lt;- formz.limited_list(formz.limit_at_most(4), field(&quot;name&quot;), definitions.text_field())\n  // names is a List(String)\n  formz.create_form(name)\n }\nSynonyms:\nformz.limited_list\nformz limited_list","url":"formz.html#limited_list"},{"doc":"formz","title":"list","content":"pub fn list(\n  config: Config,\n  is definition: Definition(a, b, c),\n  next next: fn(List(b)) -&gt; Form(a, d),\n) -&gt; Form(a, d)\n Add a list field to a form, but with no limits on the number of values that\n can be submitted. A list field is like a normal field except a consumer can\n submit multiple values, and it will return a `List` of the parsed values.\n\n The final argument is a callback that will be called when the form\n is being... constructed to look for more fields; validated to check for\n errors; and decoded to parse the input data.  **For this reason, the\n callback should be a function without side effects.** It can be called any\n number of times. Don&#39;t do anything but create the type with the data you\n need!  If you need to do decoding that has side effects, you should use\n `decode_then_try`.\nSynonyms:\nformz.list\nformz list","url":"formz.html#list"},{"doc":"formz","title":"listfield_errors","content":"pub fn listfield_errors(\n  form: Form(a, b),\n  name: String,\n  errors: List(Result(Nil, String)),\n) -&gt; Form(a, b)\n Convenience function for setting the `InputState`s of a list field. This\n takes a list of Results, where the Ok means the input is `Valid` and\n `Error` means the input is `Invalid` with the given error message.\n\n This does not clear any existing errors, it will just set the errors marked\n in the input list.  If you want to clear errors you&#39;ll have to use the\n `update` function and do it manually.\n\n ### Example\n\n \n listfield_errors(form, &quot;pet_names&quot;,  [Ok(Nil), Ok(Nil), Error(&quot;Must be a cat&quot;)])\n \nSynonyms:\nformz.listfield_errors\nformz listfield_errors","url":"formz.html#listfield_errors"},{"doc":"formz","title":"make_disabled","content":"pub fn make_disabled(config: Config) -&gt; Config\n Mark the form `Item` as disabled.  This will prevent the user from\n interacting with the field.  If you do this for a subform, it will\n only work if the form generator renders the subform as a `fieldset`.\n For example, HTML does not allow you to mark inputs in a `&lt;div&gt;`\n disabled as group but you can do this with a `&lt;fieldset&gt;`.\nSynonyms:\nformz.make_disabled\nformz make_disabled","url":"formz.html#make_disabled"},{"doc":"formz","title":"named","content":"pub fn named(name: String) -&gt; Config\n Create a field with the given name.\n\n It uses\n [justin.sentence_case](https://hexdocs.pm/justin/justin.html#sentence_case)\n to create an initial label. You can override the label with the `set_label`\n function. I don&#39;t know if this is very english-centric, so let me know if\n this is a bad experience in other languages and I&#39;ll consider\n something else.\n\n \n field(&quot;name&quot;)\n |&gt; set_label(&quot;Full Name&quot;)\n \nSynonyms:\nformz.named\nformz named","url":"formz.html#named"},{"doc":"formz","title":"required_field","content":"pub fn required_field(\n  config: Config,\n  is definition: Definition(a, b, c),\n  next next: fn(b) -&gt; Form(a, d),\n) -&gt; Form(a, d)\n Add a required field to a form.\n\n Ultimately whether a field is actually required or not comes down to the\n details of the definition.  The definition will receive the raw input string\n and is in charge of returning an error or a value.\n\n If multiple values are submitted for this field, the last one will be used.\n\n The final argument is a callback that will be called when the form\n is being... constructed to look for more fields; validated to check for\n errors; and decoded to parse the input data.  **For this reason, the\n callback should be a function without side effects.** It can be called any\n number of times. Don&#39;t do anything but create the type with the data you\n need!  If you need to do decoding that has side effects, you should use\n `decode_then_try`.\nSynonyms:\nformz.required_field\nformz required_field","url":"formz.html#required_field"},{"doc":"formz","title":"set_disabled","content":"pub fn set_disabled(config: Config, disabled: Bool) -&gt; Config\n Set the `disabled` flag directly.\nSynonyms:\nformz.set_disabled\nformz set_disabled","url":"formz.html#set_disabled"},{"doc":"formz","title":"set_help_text","content":"pub fn set_help_text(config: Config, help_text: String) -&gt; Config\n Additional instructions or help text to display to the user about the field.\nSynonyms:\nformz.set_help_text\nformz set_help_text","url":"formz.html#set_help_text"},{"doc":"formz","title":"set_label","content":"pub fn set_label(config: Config, label: String) -&gt; Config\n Set the label of the field.  This is the primary text that will be displayed\n to the user about the field.\nSynonyms:\nformz.set_label\nformz set_label","url":"formz.html#set_label"},{"doc":"formz","title":"set_name","content":"pub fn set_name(config: Config, name: String) -&gt; Config\n Set the name of the field.  This is the key that will be used for the data.\nSynonyms:\nformz.set_name\nformz set_name","url":"formz.html#set_name"},{"doc":"formz","title":"simple_limit_check","content":"pub fn simple_limit_check(\n  min: Int,\n  max: Int,\n  extra: Int,\n) -&gt; fn(Int) -&gt; Result(List(InputState), Int)\n Convenience function for creating a `LimitCheck` function.  This takes\n the minimum number of required values, the maximum number of allowed values,\n and the number of &quot;extra&quot; blank inputs that should be offered to the user\n for filling out.\n\n If &quot;extra&quot; is 0, (say, to manage blank fields via javascript), then this\n will show 1 blank field initially.\nSynonyms:\nformz.simple_limit_check\nformz simple_limit_check","url":"formz.html#simple_limit_check"},{"doc":"formz","title":"subform","content":"pub fn subform(\n  config: Config,\n  form: Form(a, b),\n  next: fn(b) -&gt; Form(a, c),\n) -&gt; Form(a, c)\n Add a form as a subform.  This will essentially append the fields from the\n subform to the current form, prefixing their names with the name of the\n subform.  Form generators will still see the fields as a set though, so they\n can be marked up as a group for accessibility reasons.\n\n The final argument is a callback that will be called when the form\n is being... constructed to look for more fields; validated to check for\n errors; and decoded to parse the input data.  **For this reason, the\n callback should be a function without side effects.** It can be called any\n number of times. Don&#39;t do anything but create the type with the data you\n need!  If you need to do decoding that has side effects, you should use\n `decode_then_try`.\nSynonyms:\nformz.subform\nformz subform","url":"formz.html#subform"},{"doc":"formz","title":"update","content":"pub fn update(\n  form: Form(a, b),\n  name: String,\n  fun: fn(Item(a)) -&gt; Item(a),\n) -&gt; Form(a, b)\n Update the [`Item`](https://hexdocs.pm/formz/formz.html#Item) with\n the given name using the provided function.  If multiple items have the same\n name, it will be called on all of them.\nSynonyms:\nformz.update\nformz update","url":"formz.html#update"},{"doc":"formz","title":"update_config","content":"pub fn update_config(\n  form: Form(a, b),\n  name: String,\n  fun: fn(Config) -&gt; Config,\n) -&gt; Form(a, b)\n Update the `Field` [details](https://hexdocs.pm/formz/formz/field.html) with\n the given name using the provided function.  If multiple items have the same\n name, it will be called on all of them.  If no items have the given name,\n or an item with the given name exists but isn&#39;t a `Field`, this function\n will do nothing.\n\n \n let form = make_form()\n update_field(form, &quot;name&quot;, field.set_label(_, &quot;Full Name&quot;))\n \nSynonyms:\nformz.update_config\nformz update_config","url":"formz.html#update_config"},{"doc":"formz","title":"validate","content":"pub fn validate(\n  form: Form(a, b),\n  names: List(String),\n) -&gt; Form(a, b)\n Validate specific fields of the form.  This is similar to `decode`, but\n instead of returning the decoded output if there are no errors, it returns\n the valid form.  This is useful for if you want to be able to give feedback\n to the user about whether certain fields are valid or not. For example, you\n could just validate only fields that the user has interacted with.\nSynonyms:\nformz.validate\nformz validate","url":"formz.html#validate"},{"doc":"formz","title":"validate_all","content":"pub fn validate_all(form: Form(a, b)) -&gt; Form(a, b)\n Validate all the fields in the form.  This is similar to `decode`, but\n instead of returning the decoded output if there are no errors, it returns\n the valid form.  This is useful for if you want to be able to give feedback\n to the user about whether certain fields are valid or not.\nSynonyms:\nformz.validate_all\nformz validate_all","url":"formz.html#validate_all"},{"doc":"formz","title":"verify","content":"pub fn verify(\n  def: Definition(a, b, c),\n  fun: fn(b) -&gt; Result(b, String),\n) -&gt; Definition(a, b, c)\n Chain additional validation onto the `parse` function.  This is\n useful if you don&#39;t need to change the returned type, but might have\n additional constraints.  Like say, requiring a `String` to be at least\n a certain length, or that an Int must be positive.\n\n ### Example\n \n field\n   |&gt; validate(fn(i) {\n     case i &gt; 0 {\n       True -&gt; Ok(i)\n       False -&gt; Error(&quot;must be positive&quot;)\n     }\n   }),\n \nSynonyms:\nformz.verify\nformz verify","url":"formz.html#verify"},{"doc":"formz","title":"widget","content":"pub fn widget(\n  def: Definition(a, b, c),\n  widget: a,\n) -&gt; Definition(a, b, c)\n Update the widget of a definition.\nSynonyms:\nformz.widget\nformz widget","url":"formz.html#widget"},{"doc":"formz","title":"formz","content":" A form is a list of fields and a decoder function. This module uses a\n series of callbacks to construct the decoder function as the fields are\n being added to the form.  The idea is that you&#39;d have a function that\n makes the form using the `use` syntax, and then be able to use the form\n later for parsing or rendering in different contexts.\n\n You can use this cheatsheet to navigate the module documentation:\n\n &lt;table&gt;\n &lt;tr&gt;\n   &lt;td&gt;Creating a form&lt;/td&gt;\n   &lt;td&gt;\n     &lt;a href=&quot;#create_form&quot;&gt;create_form&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#required_field&quot;&gt;required_field&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#field&quot;&gt;field&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#list&quot;&gt;list&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#limited_list&quot;&gt;limited_list&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#subform&quot;&gt;subform&lt;/a&gt;\n   &lt;/td&gt;\n &lt;/tr&gt;\n &lt;tr&gt;\n   &lt;td&gt;Decoding and validating a form&lt;/td&gt;\n   &lt;td&gt;\n     &lt;a href=&quot;#data&quot;&gt;data&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#decode&quot;&gt;decode&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#decode_then_try&quot;&gt;decode_then_try&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#validate&quot;&gt;validate&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#validate_all&quot;&gt;validate_all&lt;/a&gt;\n   &lt;/td&gt;\n &lt;/tr&gt;\n &lt;tr&gt;\n   &lt;td&gt;Creating a field definition&lt;/td&gt;\n   &lt;td&gt;\n     &lt;a href=&quot;#definition&quot;&gt;definition&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#definition_with_custom_optional&quot;&gt;definition_with_custom_optional&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#verify&quot;&gt;verify&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#widget&quot;&gt;widget&lt;/a&gt;\n   &lt;/td&gt;\n &lt;/tr&gt;\n &lt;tr&gt;\n   &lt;td&gt;Creating a limited list&lt;/td&gt;\n   &lt;td&gt;\n     &lt;a href=&quot;#limit_at_least&quot;&gt;limit_at_least&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#limit_at_most&quot;&gt;limit_at_most&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#limit_between&quot;&gt;limit_between&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#simple_limit_check&quot;&gt;simple_limit_check&lt;/a&gt;\n   &lt;/td&gt;\n &lt;/tr&gt;\n &lt;tr&gt;\n   &lt;td&gt;Accessing and manipulating form items&lt;/td&gt;\n   &lt;td&gt;\n     &lt;a href=&quot;#get&quot;&gt;get&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#items&quot;&gt;items&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#field_error&quot;&gt;field_error&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#listfield_errors&quot;&gt;listfield_errors&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#update&quot;&gt;update&lt;/a&gt;&lt;br&gt;\n   &lt;/td&gt;\n &lt;/tr&gt;\n &lt;tr&gt;\n   &lt;td&gt;Accessing and manipulating config for a form item&lt;/td&gt;\n   &lt;td&gt;\n     &lt;a href=&quot;#update_config&quot;&gt;update_config&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#set_name&quot;&gt;set_name&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#set_label&quot;&gt;set_label&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#set_help_text&quot;&gt;set_help_text&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#set_disabled&quot;&gt;set_disabled&lt;/a&gt;&lt;br&gt;\n     &lt;a href=&quot;#make_disabled&quot;&gt;make_disabled&lt;/a&gt;&lt;br&gt;\n   &lt;/td&gt;\n &lt;/tr&gt;\n &lt;/table&gt;\n\n ### Examples\n\n ```gleam\n fn make_form() {\n  use name &lt;- formz.required_field(field(&quot;name&quot;), definitions.text_field())\n\n  formz.create_form(name)\n }\n\n fn process_form() {\n   make_form()\n   |&gt; formz.data([#(&quot;name&quot;, &quot;Louis&quot;))])\n   |&gt; formz.decode\n   # -&gt; Ok(&quot;Louis&quot;)\n }\n ```\n\n ```gleam\n fn make_form() {\n  use greeting &lt;- field(field(&quot;greeting&quot;), definitions.text_field())\n  use name &lt;- field(field(&quot;name&quot;), definitions.text_field())\n\n  formz.create_form(greeting &lt;&gt; &quot; &quot; &lt;&gt; name)\n }\n\n fn process_form() {\n   make_form()\n   |&gt; data([#(&quot;greeting&quot;, &quot;Hello&quot;), #(&quot;name&quot;, &quot;World&quot;)])\n   |&gt; formz.decode\n   # -&gt; Ok(&quot;Hello World&quot;)\n }\n ```","url":"formz.html"},{"doc":"formz/validation","title":"and","content":"pub fn and(\n  previous: fn(a) -&gt; Result(b, String),\n  next: fn(b) -&gt; Result(c, String),\n) -&gt; fn(a) -&gt; Result(c, String)\n Chain validations together.\n\n ## Examples\n\n \n let is_even = fn(num) {\n   case num % 2 == 0 {\n     True -&gt; Ok(num)\n     False -&gt; Error(&quot;must be even&quot;)\n   }\n }\n let check = and(int, is_even)\n\n check(&quot;2&quot;)\n # -&gt; Ok(2)\n\n check(&quot;hi&quot;)\n # -&gt; Error(&quot;must be a whole number&quot;)\n\n check(&quot;1&quot;)\n # -&gt; Error(&quot;must be even&quot;)\nSynonyms:\nformz/validation.and\nformz/validation and","url":"formz/validation.html#and"},{"doc":"formz/validation","title":"email","content":"pub fn email(input: String) -&gt; Result(String, String)\n Parse the input as a String that looks like an email address, i.e. it\n contains an `@` character, with at least one other character on either\n side\n\n (this behavior more closely matches what the browser does than just\n checking for an `@`).\n\n ## Examples\n\n \n email(&quot;hello@example.com&quot;)\n # -&gt; Ok(&quot;hello@example.com&quot;)\n \n \n email(&quot;@&quot;)\n # -&gt; Error(&quot;Must be an email address&quot;)\n \n \n email(&quot;hello&quot;)\n # -&gt; Error(&quot;Must be an email address&quot;)\n \nSynonyms:\nformz/validation.email\nformz/validation email","url":"formz/validation.html#email"},{"doc":"formz/validation","title":"int","content":"pub fn int(str: String) -&gt; Result(Int, String)\n Parse the input as an int.\n\n ## Examples\n\n \n int(&quot;1&quot;)\n # -&gt; Ok(1)\n \n\n \n int(&quot;3.4&quot;)\n # -&gt; Error(&quot;Must be a whole number&quot;)\n \n \n int(&quot;hello&quot;)\n # -&gt; Error(&quot;Must be a whole number&quot;)\n \nSynonyms:\nformz/validation.int\nformz/validation int","url":"formz/validation.html#int"},{"doc":"formz/validation","title":"list_item_by_index","content":"pub fn list_item_by_index(\n  variants: List(a),\n) -&gt; fn(String) -&gt; Result(a, String)\n Validates that the input is one from a list of allowed values. Takes a list\n of Gleam values that can be chosen.  This uses the index of the item to\n find the desired value.\n\n ## Examples\n\n \n enum([&quot;One&quot;,&quot;Two&quot;,&quot;Three&quot;])(&quot;1&quot;)\n # -&gt; Ok(&quot;Two&quot;)\n \n\n \n enum([True, False])(&quot;42&quot;)\n # -&gt; Error(&quot;must be an item in list&quot;)\n \n\n \n enum([True, False])(&quot;ok&quot;)\n # -&gt; Error(&quot;must be an item in list&quot;)\n \nSynonyms:\nformz/validation.list_item_by_index\nformz/validation list_item_by_index","url":"formz/validation.html#list_item_by_index"},{"doc":"formz/validation","title":"non_empty_string","content":"pub fn non_empty_string(str: String) -&gt; Result(String, String)\n Trim and leave the input as it is, but verify it is non-empty.\n\n \n non_empty_string(&quot;hello&quot;)\n # -&gt; Ok(&quot;hello&quot;)\n \n\n \n non_empty_string(&quot;  &quot;)\n # -&gt; Error(&quot;is required&quot;)\n \nSynonyms:\nformz/validation.non_empty_string\nformz/validation non_empty_string","url":"formz/validation.html#non_empty_string"},{"doc":"formz/validation","title":"number","content":"pub fn number(str: String) -&gt; Result(Float, String)\n Parse the input as a float.  this is forgiving and will also parse\n ints into floats.\n\n ## Examples\n\n \n number(&quot;1&quot;)\n # -&gt; Ok(1.0)\n \n\n \n number(&quot;3.4&quot;)\n # -&gt; Ok(3.4)\n \n \n number(&quot;hello&quot;)\n # -&gt; Error(&quot;Must be a number&quot;)\n \nSynonyms:\nformz/validation.number\nformz/validation number","url":"formz/validation.html#number"},{"doc":"formz/validation","title":"on","content":"pub fn on(val: String) -&gt; Result(Bool, String)\n Parse the input as a boolean, where only &quot;on&quot; is True and allowed.\n All other values are an error.  This is useful for HTML checkboxes, which\n the browser sends the empty string if unchecked, and `&quot;on&quot;` if  checked.\n\n ## Examples\n\n \n on(&quot;on&quot;)\n # -&gt; Ok(True)\n \n\n \n on(&quot;&quot;)\n # -&gt; Error(&quot;is required&quot;)\n \n\n \n on(&quot;hi&quot;)\n # -&gt; Error(&quot;is required&quot;)\n \nSynonyms:\nformz/validation.on\nformz/validation on","url":"formz/validation.html#on"},{"doc":"formz/validation","title":"replace_error","content":"pub fn replace_error(\n  previous: fn(a) -&gt; Result(b, String),\n  error: String,\n) -&gt; fn(a) -&gt; Result(b, String)\n Replace the error message of a validation with a new one.  Most of the built-in\n error messages are pretty rudimentary.\nSynonyms:\nformz/validation.replace_error\nformz/validation replace_error","url":"formz/validation.html#replace_error"},{"doc":"formz/validation","title":"formz/validation","content":"","url":"formz/validation.html"}]);